/************************
  	mergblk.c
		2013.9.18
		By H.Nakano
	version 0.1
	License GPL version 3
****************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <stdarg.h>
#include <errno.h>
#include <getopt.h>

#include "bmerge.h"

static iprange_t *Brange;
static int Bsize = 0;
static int Bcount = 0;

void usage(){
	printf("usage: mergblk -o Black.txt blacklist1.txt blacklist2.txt ... \n");
	printf("\t -o,--outfile  output filename( '-' is stdout)\n");

	printf("\tex)\n");
	printf("\tmergblk -o Black1.txt Blacklists/blacklist1.txt\n");
	printf("\tmergblk -o Black2.txt Blacklists/blacklist1.txt Blacklists/blacklist2.txt\n");
	printf("\tmergblk -o - Blacklists/blacklist1.txt\n");
	exit(1);
}

/* read_file.c */
extern ipaddr_range_t *read_file(FILE *fp, int *val);
/* print_addr.c */
extern int  check_dup(ipaddr_range_t *x,int size);
extern void qsort2(ipaddr_range_t *x,uint32_t left,uint32_t right);
/* addr_tools.c */
extern void num2addr(uint32_t num,int range,char *val);

int save_black(char *filename){
	int  ret = 0;
	char addr[32];
	FILE *fp;
	ipaddr_range_t * val;

	val = (ipaddr_range_t *)malloc(Bsize*sizeof(ipaddr_range_t));

	int j = 0;
	for(int i=0;i < Bcount;i++){
		memcpy((void *)&val[j],(void *)Brange[i].dat,Brange[i].size*sizeof(ipaddr_range_t));
		free(Brange[i].dat);
		j += Brange[i].size;
	}

	qsort2(val,0,Bsize -1);
	Bsize = check_dup(val,Bsize);

	printf("Total: number of line = %d\n",Bsize);
	errno = 0;
	if (filename[0] == '-') 
		fp = stdout;
	else{
		fp = fopen(filename,"w");
		if (fp == NULL){
			printf("Error: open (%s) (%s)\n",filename,strerror(errno));
			return -1;
		 }
	 }
	for(int i=0;i < Bsize;i++){
		num2addr(val[i].num,val[i].range,addr);
		ret = fwrite((void *)addr,strlen(addr),1,fp );
		if (ret <= 0)
		  	break;
	 }
	fclose(fp);
	free(val);
	if (ret <= 0)
	   unlink(filename);
	return 0;
}

int set_blacklist(char *filename){
	int  size;
	ipaddr_range_t *val;
	FILE *fp;

	errno = 0;
	fp = fopen(filename,"r");
	if (fp ==NULL){
		printf("Error: open (%s) %s\n",filename,strerror(errno));
		return -1;
	 }

	val = read_file(fp,&size);
	errno = 0;
	if (fclose(fp) != 0){
		perror("Error: fclose");
	//	return -1;
	 }

	if (val == NULL){
		if (size == 0)
			printf("(%s) non data\n",filename);
		return 0;
	 } 
	printf("%s (%d)\n",filename,size);
	Brange[Bcount].size = size;
	Brange[Bcount].dat = val;
	Bcount++;
	Bsize += size;
	return 0;
}

int main(int argc,char *argv[]){
  	int ret;
	char *filename = NULL;

	if (argc <= 3)
		usage();

	const struct option longopt[] = {
    	{"help", no_argument, NULL, '?'},
    	{"outfile",required_argument,NULL, 'o'},
    	{NULL, 0, NULL, 0}
     };

  	int c;
  	opterr = 0;
 	while ((c = getopt_long(argc, argv, "o:",longopt,NULL)) != -1){
     switch (c)
       {
     	case 'o': 
			filename = optarg;
        	break;
		default:
			usage();
			break;
	   }
	}

	if (filename == NULL){
		printf("-o output_filename : NULL\n\n");
		usage();
	}

	Brange = (iprange_t *)malloc((argc - optind)*sizeof(iprange_t));
	for(int i=optind; i<argc;i++){
		ret = set_blacklist(argv[i]);
		if (ret < 0){
			printf("error:(%s)\n",argv[i]);
			free(Brange);
			return -1;
		 }
	 }
	if (Bcount > 0)
		save_black(filename);
	free(Brange);
	return 0;
}


