#!/usr/bin/env lua
--
--	mk_black.lua
--		2013.10.11
--		By H.Nakano
--		License GPL version 3

require("strict")
local tablex  = require("pl.tablex")
local stringx = require("pl.stringx")
local lapp   = require("pl.lapp")
local path   = require("pl.path")
local Date   = require("pl.Date")
--local pretty = require("pl.pretty")

local Version = 0.1
-- usage --
local usable_args = string.format([[
Usage: mk_black.lua Options  log_file
 mk_black.lua is a lua script that abstracts black-ipaddresses from log_files.

 Options
	-m,--mode (default from) [from|to]
	-n,--name (default "all")  (wrong_servers,invalid_servers,...)
	-o,--out  (default blacklist.txt) 
	-v,--version  print version
	-h,--help     print this
	<files...>  (default "")

	ex)
	  mk_black.lua -m from -o black01.txt nfqd_1_2013_1011.log nfqd_1_2013_1012.log
	  mk_black.lua -m from -o black01.txt /var/log/iptips/nfqd_2_2013_1101.log /var/log/iptips/nfqd_1_2013_1101.log

	  mk_black.lua -m from -n wrong_servers,invalid_servers -o black01.txt nfqd_1_2013_1011.log nfqd_1_2013_1012.log
	  input-format)
		2013-10-12 16:57:07 DROP (wrong_servers) from=211.189.116.219 to=xx.xx.xx.xx

	version %s
		2013.10.12
		By H.Nakano 
]],Version)

local function usage()
	print(usable_args)
	os.exit()
end

local Val = {}

local function write_val(filename)
	local fp,err = io.open(filename,"w")
	if fp == nil then
        print(string.format("Error: file open (%s)",filename) ,err)
   		 os.exit()
	end

	fp:write(string.format("# %s\n",tostring(Date())))
	fp:write(string.format("# %s\n",stringx.join(' ',args.files)))
	table.sort(Val)
	for _,v in ipairs(Val) do
		fp:write(v,"\n")
	end
	
	io.close(fp)
end

local function check_dat(dat)
	local f,addr = stringx.splitv(dat,'=')
	if addr then
		if tablex.find(Val,addr) == nil then
			--	print(addr)
			table.insert(Val,addr)
		end
	end
end

local function check_name(dat)
	if args.name == "all" then
		return true
	end
	for _,v in ipairs(args.name) do
		if dat == v then
			return true
		end
	end
	return false
end

local function set_black(filename)
	local fp,err = io.open(filename,"r")
	if fp == nil then
        print(string.format("Error: file open (%s)",filename) ,err)
   		return
	end

	for line in fp:lines() do
		local dat = stringx.split(line," ",7)
		if dat and tablex.size(dat) >= 6 and check_name(dat[4]) then
			if args.mode == "from" and stringx.startswith(dat[5],"From=") then
				check_dat(dat[5])
			elseif args.mode == "to" and stringx.startswith(dat[6],"To=") then
				check_da(dat[6])
			end
		end
	end
	io.close(fp)
end

-- ### --
if arg[1] == nil then
    usage()
else
	args = lapp(usable_args)

	if args.version == true then
		print(string.format(" Version %s",Version))
		os.exit()
	end

	if args.name ~= "all" then
		local nn = stringx.split(args.name,',')
		args.name = {}
		for _,v in ipairs(nn) do
			table.insert( args.name , string.format("(%s)",v))
		end
	end	

	-- pretty.dump(args)

	for n,v in ipairs(args.files) do
		if path.isfile(v) then
			set_black(v)
		else
			 print(string.format("Error: couldn't be found (%s)",v))
		end
	end

	if tablex.size(Val) >= 1 then
		 write_val(args.out)
	else
		print("no data")
	end
end

